@@ -421,8 +421,6 @@ def _duplicate_node(model, existing_node, add_as_part = True):
         uuid = rqet.uuid_for_part_root(new_node)
         if model.part_for_uuid(uuid) is None:
             _add_part(model, rqet.node_type(new_node), uuid, new_node)
-        else:
-            log.warning('rejected attempt to add a duplicated part with an existing uuid')
     return new_node
 
 
@@ -451,7 +449,7 @@ def _copy_part_from_other_model(model,
 
     assert other_model is not None
     if other_model is model:
-        return
+        return part
     assert part is not None
     if realization is not None:
         assert isinstance(realization, int) and realization >= 0
@@ -465,13 +463,13 @@ def _copy_part_from_other_model(model,
 
     # check whether already existing in this model
     if part in model.parts_forest.keys():
-        return
+        return part
 
     if other_model.type_of_part(part) == 'obj_EpcExternalPartReference':
-        log.debug('refusing to copy hdf5 ext part from other model')
-        return
+        # log.debug('refusing to copy hdf5 ext part from other model')
+        return None
 
-    log.debug('copying part: ' + str(part))
+    # log.debug('copying part: ' + str(part))
 
     uuid = rqet.uuid_in_part_name(part)
     if not force:
@@ -481,7 +479,7 @@ def _copy_part_from_other_model(model,
     other_root = other_model.root_for_part(part, is_rels = False)
     if other_root is None:
         log.error('failed to copy part (missing in source model?): ' + str(part))
-        return
+        return None
 
     resident_uuid = _unforced_consolidation(model, other_model, consolidate, force, part)
 
@@ -520,6 +518,8 @@ def _copy_part_from_other_model(model,
     # copy relationships where target part is present in this model – this part is source, then destination
     _copy_relationships_for_present_targets(model, other_model, consolidate, force, resident_uuid, root_node)
 
+    return model.part_for_uuid(resident_uuid)
+
 
 def _unforced_consolidation(model, other_model, consolidate, force, part):
     if consolidate and not force:
@@ -659,6 +659,7 @@ def _copy_all_parts_from_other_model(model, other_model, realization = None, con
 def _set_uuid_to_part(model, part_name):
     """Adds an entry to the dictionary mapping from uuid to part name."""
 
+    assert part_name and isinstance(part_name, str)
     uuid = rqet.uuid_in_part_name(part_name)
     model.uuid_part_dict[bu.uuid_as_int(uuid)] = part_name
 
