@@ -1,11 +1,10 @@
 """model.py: Main resqml interface module handling epc packing & unpacking and xml structures."""
 
-version = '1st December 2021'
+version = '15th December 2021'
 
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('model.py version ' + version)
 
 import h5py
 import os
@@ -17,6 +16,10 @@ import resqpy.model._forestry as m_f
 import resqpy.model._grids as m_g
 import resqpy.model._hdf5 as m_h
 import resqpy.model._xml as m_x
+import resqpy.olio.xml_et as rqet
+from resqpy import __version__
+
+log.debug('resqpy Model class version: ' + __version__ + '; xml citation format: ' + m_x.citation_format)
 
 
 class Model():
@@ -486,7 +489,7 @@ class Model():
 
     @property
     def crs_root(self):
-        """XML node corresponding to self.crs_uuid."""
+        """XML node corresponding to self.crs_uuid, the 'main' crs for the model."""
 
         return self.root_for_uuid(self.crs_uuid)
 
@@ -774,6 +777,18 @@ class Model():
 
         return m_c._uuid_for_part(self, part_name, is_rels = is_rels)
 
+    def uuid_for_root(self, root_node):
+        """Returns the uuid for an object given an xml root node.
+
+        arguments:
+           root_node: the xml root node for the object for which the uuid is required
+
+        returns:
+           uuid.UUID for the specified object
+        """
+
+        return rqet.uuid_for_part_root(root_node)
+
     def type_of_part(self, part_name, strip_obj = False):
         """Returns content type for the named part (does not apply to rels parts).
 
@@ -1479,60 +1494,10 @@ class Model():
 
         return m_x._create_doc_props(self, add_as_part = add_as_part, root = root, originator = originator)
 
-    def create_crs(self,
-                   add_as_part = True,
-                   title = 'cell grid local CRS',
-                   epsg_code = None,
-                   originator = None,
-                   x_offset = 0.0,
-                   y_offset = 0.0,
-                   z_offset = 0.0,
-                   areal_rotation_radians = 0.0,
-                   xy_units = 'm',
-                   z_units = 'm',
-                   z_inc_down = True):
-        """DEPRECATED: Creates a Coordinate Reference System node and optionally adds as child of root and/or to parts forest.
-
-        arguments:
-            add_as_part (boolean, default True): if True the newly created crs node is added to the model
-            as a part
-            title (string): used as the Title text in the citation node
-            epsg_code (integer): EPSG code of the parent coordinate reference system that this crs sits within;
-            used for both projected and vertical frames of reference; if None then unknown settings are used
-            originator (string, optional): the name of the human being who created the crs object;
-            default is to use the login name
-            x_offset, y_offset, z_offset (floats, default 0.0): the local origin within the parent coordinate
-            reference system space
-            areal_rotation_radians (float, default 0.0): the areal rotation of the xy axes of this crs relative
-            to the parent coordinate reference system
-            xy_units (string, default 'm'): the length units of x & y values in this crs; 'm' or 'ft'
-            z_units (string, default 'm'): the length units of z values in this crs; 'm' or 'ft'
-            z_inc_down (boolean, default True): if True, z values increase with depth; if False, z values increase
-            with elevation
-
-        returns:
-            newly created coordinate reference system xml node
-        """
-
-        warnings.warn("Model.create_crs() is DEPRECATED, will be removed", DeprecationWarning)
-        return m_x._create_crs(self,
-                               add_as_part = add_as_part,
-                               title = title,
-                               epsg_code = epsg_code,
-                               originator = originator,
-                               x_offset = x_offset,
-                               y_offset = y_offset,
-                               z_offset = z_offset,
-                               areal_rotation_radians = areal_rotation_radians,
-                               xy_units = xy_units,
-                               z_units = z_units,
-                               z_inc_down = z_inc_down)
-
-    def create_crs_reference(self, crs_root = None, root = None, crs_uuid = None):
+    def create_crs_reference(self, root = None, crs_uuid = None):
         """Creates a node refering to an existing crs node and optionally adds as child of root.
 
         arguments:
-           crs_root: DEPRECATED, use uuid instead; the root xml node for the coordinate reference system being referenced
            root: the xml node to which the new reference node is to appended as a child (ie. the xml node
               for the object that is referring to the crs)
            crs_uuid: the uuid of the crs
@@ -1541,7 +1506,7 @@ class Model():
            newly created crs reference xml node
         """
 
-        return m_x._create_crs_reference(self, crs_root = crs_root, root = root, crs_uuid = crs_uuid)
+        return m_x._create_crs_reference(self, root = root, crs_uuid = crs_uuid)
 
     def create_md_datum_reference(self, md_datum_root, root = None):
         """Creates a node refering to an existing measured depth datum and optionally adds as child of root.
@@ -1755,13 +1720,17 @@ class Model():
            rel_type_a (string): the Type (role) associated with node_a in the relationship;
               usually 'sourceObject' or 'destinationObject'
            node_b: the other xml node to be related
-           rel_type_a (string): the Type (role) associated with node_b in the relationship
+           rel_type_b (string): the Type (role) associated with node_b in the relationship
               usually 'sourceObject' or 'destinationObject' (opposite of rel_type_a)
            avoid_duplicates (boolean, default True): if True, xml for a relationship is not added
               where it already exists; if False, a duplicate will be created in this situation
 
         returns:
            None
+
+        note:
+           this method has the same effect as create_reciprocal_relationship_uuids() but takes xml root nodes
+           rather than uuids as arguments
         """
 
         return m_x._create_reciprocal_relationship(self,
@@ -1771,6 +1740,37 @@ class Model():
                                                    rel_type_b,
                                                    avoid_duplicates = avoid_duplicates)
 
+    def create_reciprocal_relationship_uuids(self, uuid_a, rel_type_a, uuid_b, rel_type_b, avoid_duplicates = True):
+        """Adds a node to each of a pair of trees in the rels forest, to represent a two-way relationship.
+
+        arguments:
+           uuid_a: uuid of one of the two parts to be related
+           rel_type_a (string): the Type (role) associated with uuid_a in the relationship;
+              usually 'sourceObject' or 'destinationObject'
+           uuid_b: uuid of the other part to be related
+           rel_type_b (string): the Type (role) associated with uuid_b in the relationship
+              usually 'sourceObject' or 'destinationObject' (opposite of rel_type_a)
+           avoid_duplicates (boolean, default True): if True, xml for a relationship is not added
+              where it already exists; if False, a duplicate will be created in this situation
+
+        returns:
+           None
+
+        note:
+           this method has the same effect as create_reciprocal_relationship() but takes uuids rather than
+           xml root nodes as arguments
+        """
+
+        node_a = self.root_for_uuid(uuid_a)
+        node_b = self.root_for_uuid(uuid_b)
+        assert node_a is not None and node_b is not None, 'part not present when creating relationship'
+        return m_x._create_reciprocal_relationship(self,
+                                                   node_a,
+                                                   rel_type_a,
+                                                   node_b,
+                                                   rel_type_b,
+                                                   avoid_duplicates = avoid_duplicates)
+
     def duplicate_node(self, existing_node, add_as_part = True):
         """Creates a deep copy of the xml node (typically from another model) and optionally adds as part.
 
@@ -1787,7 +1787,9 @@ class Model():
            duplicated;
            the uuid of the part is not changed by this function, so if the source and target models are
            the same, add as part should be set False and calling code will need to assign a new uuid
-           prior to adding as part
+           prior to adding as part;
+           if add_as_part is True, the part is only added if the uuid does not already exist in this
+           model
         """
 
         return m_f._duplicate_node(self, existing_node, add_as_part = add_as_part)
@@ -1830,22 +1832,86 @@ class Model():
            other_h5_file_name (string, optional): h5 file name for other model; can be passed as
               an optimisation when calling method repeatedly
 
+        returns:
+           the part name of the part in this model, after copying; may differ from requested part if
+           consolidate is True; None in the case of failure
+
         notes:
            if the part name already exists in this model, no action is taken;
            default hdf5 file used in this model and assumed in other_model
         """
 
-        m_f._copy_part_from_other_model(self,
-                                        other_model,
-                                        part,
-                                        realization = realization,
-                                        consolidate = consolidate,
-                                        force = force,
-                                        cut_refs_to_uuids = cut_refs_to_uuids,
-                                        cut_node_types = cut_node_types,
-                                        self_h5_file_name = self_h5_file_name,
-                                        h5_uuid = h5_uuid,
-                                        other_h5_file_name = other_h5_file_name)
+        return m_f._copy_part_from_other_model(self,
+                                               other_model,
+                                               part,
+                                               realization = realization,
+                                               consolidate = consolidate,
+                                               force = force,
+                                               cut_refs_to_uuids = cut_refs_to_uuids,
+                                               cut_node_types = cut_node_types,
+                                               self_h5_file_name = self_h5_file_name,
+                                               h5_uuid = h5_uuid,
+                                               other_h5_file_name = other_h5_file_name)
+
+    def copy_uuid_from_other_model(self,
+                                   other_model,
+                                   uuid,
+                                   realization = None,
+                                   consolidate = True,
+                                   force = False,
+                                   cut_refs_to_uuids = None,
+                                   cut_node_types = None,
+                                   self_h5_file_name = None,
+                                   h5_uuid = None,
+                                   other_h5_file_name = None):
+        """Fully copies part for uuid in from another model, with referenced parts, hdf5 data and relationships.
+
+        arguments:
+           other model (Model): the source model from which to copy a part
+           uuid (UUID): the uuid of the part in the other model to copy into this model
+           realization (int, optional): if present and the part is a property, the realization
+              will be set to this value, instead of the value in use in the other model if any
+           consolidate (boolean, default True): if True and an equivalent part already exists in
+              this model, do not duplicate but instead note uuids as equivalent
+           force (boolean, default False): if True, the part itself is copied without much checking
+              and all references are required to be handled by an entry in the consolidation object
+           cut_refs_to_uuids (list of UUIDs, optional): if present, then xml reference nodes
+              referencing any of the listed uuids are cut out in the copy; use with caution
+           cut_node_types (list of str, optional): if present, any child nodes of a type in the list
+              will be cut out in the copy; use with caution
+           self_h5_file_name (string, optional): h5 file name for this model; can be passed as
+              an optimisation when calling method repeatedly
+           h5_uuid (uuid, optional): UUID for this model's hdf5 external part; can be passed as
+              an optimisation when calling method repeatedly
+           other_h5_file_name (string, optional): h5 file name for other model; can be passed as
+              an optimisation when calling method repeatedly
+
+        returns:
+           the uuid of the part in this model, after copying; may differ from requested uuid if
+           consolidate is True; None in the case of failure
+
+        notes:
+           if the part already exists in this model, no action is taken;
+           default hdf5 file used in this model and assumed in other_model
+        """
+
+        part = other_model.part_for_uuid(uuid)
+        if part is None:
+            return None
+        copied_part = m_f._copy_part_from_other_model(self,
+                                                      other_model,
+                                                      part,
+                                                      realization = realization,
+                                                      consolidate = consolidate,
+                                                      force = force,
+                                                      cut_refs_to_uuids = cut_refs_to_uuids,
+                                                      cut_node_types = cut_node_types,
+                                                      self_h5_file_name = self_h5_file_name,
+                                                      h5_uuid = h5_uuid,
+                                                      other_h5_file_name = other_h5_file_name)
+        if copied_part is None:
+            return None
+        return self.uuid_for_part(copied_part)
 
     def copy_all_parts_from_other_model(self, other_model, realization = None, consolidate = True):
         """Fully copies parts in from another model, with referenced parts, hdf5 data and relationships.
@@ -1893,7 +1959,7 @@ class Model():
     def iter_grid_connection_sets(self):
         """Yields grid connection set objects, one for each gcs in this model."""
 
-        for gcs in m_c._iter_grid_connection_sets(self):
+        for gcs in list(m_c._iter_grid_connection_sets(self)):
             yield gcs
 
     def iter_wellbore_interpretations(self):
