@@ -7,7 +7,6 @@ version = '29th April 2021'
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('load_data.py version %s', version)
 
 # defs:
 #    def file_exists(file_name):
@@ -27,7 +26,6 @@ import resqpy.olio.ab_toolbox as abt
 import resqpy.olio.box_utilities as box
 import resqpy.olio.grid_functions as gf
 import resqpy.olio.keyword_files as kf
-import resqpy.olio.utility as pu
 import resqpy.olio.write_data as wd
 
 
@@ -39,8 +37,8 @@ def file_exists(file_name, must_be_more_recent_than_file = None):
     existe = os.path.exists(file_name)
     if not existe:
         return False
-    if not must_be_more_recent_than_file or file_name == must_be_more_recent_than_file or  \
-       not os.path.exists(must_be_more_recent_than_file):
+    if not must_be_more_recent_than_file or file_name == must_be_more_recent_than_file or \
+            not os.path.exists(must_be_more_recent_than_file):
         return True
     return os.path.getmtime(file_name) > os.path.getmtime(must_be_more_recent_than_file)
 
@@ -182,7 +180,7 @@ def load_array_from_file(file_name,
         cell_count = -1  # np.fromfile interprets this as 'read everything'
         log.debug('Loading unknown number of array data elements from file ' + file_name)
     else:
-        cell_count = pu.cell_count_from_extent(extent)
+        cell_count = np.product(extent)
         log.debug('Loading %1d array data elements from file %s', cell_count, file_name)
 
     ascii_file_name = file_name
@@ -297,7 +295,7 @@ def load_array_from_ascii_file(file_name,
         cell_count = -1  # np.fromfile interprets this as 'read everything'
         log.debug('Loading unknown number of array data elements from ascii file ' + file_name)
     else:
-        cell_count = pu.cell_count_from_extent(extent)
+        cell_count = np.product(extent)
         log.debug('Loading %1d array data elements from ascii file %s', cell_count, file_name)
 
     if data_type in ['real', 'float', float]:
@@ -427,117 +425,3 @@ def load_array_from_ascii_file(file_name,
 
 # end of load_array_from_ascii_file() def
 ######################################################################################################
-
-######################################################################################################
-# load_fault_mask():
-# function to load fault indicator data for cell faces from nexus input deck format ascii file
-# indicator array is a numpy array with data type 'bool' and must have been initialised to false
-# only faces specified in the input file are modified
-# the indicator array is set to true for every face mentioned in the MULT data, regardless of value
-# binary data file creation & reuse is supported
-
-
-def load_fault_mask(file_name, direction, fault_mask, use_binary = False):
-    """Modifies a 3D numpy boolean array representing fault locations on cell faces in i,j or k.
-
-    arguments:
-       file_name: string giving name of existing ascii file holding nexus MULT keywords in input deck format
-       direction: a single character giving direction of interest, must be 'i', 'j' or 'k'
-       fault_mask: an existing 3D boolean numpy array which must have been initialised to False everywhere
-       use_binary: if True, will attempt to load the boolean mask from a pre-written pure binary file,
-                  which will be created if it does not exist
-
-    returns:
-       None
-
-    notes :
-       see nexus keyword reference for format of MULT keyword
-
-       fault_mask element is set to True if corresponding face appears in the MULT data set,
-       whatever function is being applied, and whatever value is specified the function returns None (nothing)
-    """
-
-    trans_keyword_dict = {'i': 'TX', 'j': 'TY', 'k': 'TZ'}
-
-    assert (direction in 'ijk')
-    assert (fault_mask.ndim == 3)
-    mask_extent_kji = fault_mask.shape
-    ascii_file_name = file_name
-    binary_file_name = file_name
-
-    face_count = pu.cell_count_from_extent(mask_extent_kji)
-    log.debug('Loading %1d fault mask elements from file %s', face_count, file_name)
-
-    if use_binary:
-        # try reading mask from binary file
-        if len(binary_file_name) > 5 and binary_file_name[-5:] == '.' + direction.lower() + '.bb':
-            ascii_file_name = ascii_file_name[:-5]
-        else:
-            binary_file_name = binary_file_name + '.' + direction.lower() + '.bb'
-        try:
-            with open(binary_file_name, 'rb') as binary_data_file:
-                result = np.fromfile(binary_data_file, dtype = 'bool', count = face_count).reshape(mask_extent_kji)
-                fault_mask.data = result.data
-                # todo: check that end of file has been reached, ie. not too much data in file
-                log.info('Fault mask data loaded from binary file %s', binary_file_name)
-                return
-        except Exception:
-            pass
-
-    with open(ascii_file_name, 'r') as data_file:
-
-        while kf.find_keyword_pair(data_file, 'MULT', trans_keyword_dict[direction]):
-
-            line = data_file.readline().upper()
-            line = line.partition('!')[0]  # removes trailing comment
-            plus_minus_offset = [0, 0, 0]  # only possibly set to 1 in direction of interest
-            if ('MINUS' not in line.split()):  # indicates data apply to 'negative' faces of specified cells
-                plus_minus_offset['kji'.index(direction)] = 1
-
-            if not kf.find_keyword(data_file, 'FNAME'):
-                continue  # will mess up if not present for this MULT
-            line = data_file.readline().upper()
-            words = line.split()
-
-            if not kf.find_number(data_file):
-                continue  # will mess up if no numbers before next MULT
-
-            count = 0
-            while kf.number_next(data_file):
-                # fetch a new box from the input file (usually the box contains just one cell)
-                line = data_file.readline().upper()
-                line = line.partition('!')[0]  # removes trailing comment
-                words = line.split()
-                box_kji0 = box.box_kji0_from_words_iijjkk1(words)
-                box_kji0[0] = box_kji0[0] + plus_minus_offset  # adjust min indices
-                box_kji0[1] = box_kji0[1] + plus_minus_offset  # adjust max indices
-                # check that box is within extent of fault_mask
-                for dir in range(2):
-                    assert (box_kji0[0, dir] >= 0)
-                    assert (box_kji0[1, dir] < mask_extent_kji[dir])
-                # set fault_mask to True for face of every cell in box (usually just one cell, or column)
-#            for k in range(box_kji0[0, 0], box_kji0[1, 0] + 1):
-#               for j in range(box_kji0[0, 1], box_kji0[1, 1] + 1):
-#                  for i in range(box_kji0[0, 2], box_kji0[1, 2] + 1):
-#                     if not fault_mask[k,j,i]:
-#                        fault_mask[k, j, i] = True
-#                        count += 1
-                count += (box.volume_of_box(box_kji0) -
-                          np.count_nonzero(fault_mask[box_kji0[0, 0]:box_kji0[1, 0] + 1, box_kji0[0, 1]:box_kji0[1, 1] +
-                                                      1, box_kji0[0, 2]:box_kji0[1, 2] + 1]))
-                fault_mask[box_kji0[0, 0]:box_kji0[1, 0] + 1, box_kji0[0, 1]:box_kji0[1, 1] + 1,
-                           box_kji0[0, 2]:box_kji0[1, 2] + 1] = True
-
-    if use_binary:
-        # create binary file for use next time
-        try:
-            with open(binary_file_name, 'wb') as binary_file_out:
-                fault_mask.tofile(binary_file_out)
-            log.info('Binary fault mask data file %s created', binary_file_name)
-        except Exception:
-            log.warn('Failed to write data to binary fault mask file %s', binary_file_name)
-            # todo: could delete the binary file in case a corrupt file is left for use next time
-
-
-# end of load_fault_mask() def
-######################################################################################################
