@@ -5,7 +5,6 @@ version = '29th April 2021'
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('point_inclusion.py version ' + version)
 
 import math as maths
 import os
@@ -105,6 +104,8 @@ def pip_array_cn(p_a, poly):
     # returned boolean array has shape of p_array less the final axis
 
     elements = np.prod(list(p_a.shape)[:-1], dtype = int)
+    if elements == 0:
+        return np.zeros((0,), dtype = bool)
     p = p_a.reshape((elements, -1))
     crossings = np.zeros((elements,), dtype = int)
 
