@@ -7,7 +7,6 @@ version = '19th January 2021'
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('simple_lines.py version ' + version)
 
 import numpy as np
 
@@ -279,7 +278,7 @@ def drape_lines_to_rods(line_list,
     assert axis.upper() in ['I', 'J']
     assert len(line_list) == len(rod_list_list)
 
-    x_sect = grid.x_section_points(axis, ref_slice0 = ref_slice0, plus_face = plus_face)
+    x_sect = grid.unsplit_x_section_points(axis, ref_slice0 = ref_slice0, plus_face = plus_face)
 
     draped_list = []
     for line, rod_list in zip(line_list, rod_list_list):
