@@ -1,4 +1,8 @@
-"""Module defining dictionary of nexus WELLSPEC column keywords"""
+"""Module for loading WELLSPEC files. 
+
+The module includes a dictionary of nexus WELLSPEC column keywords, functionality to
+read WELLSPEC files and transform the well data into Pandas DataFrames.
+"""
 
 version = "19th April 2022"
 
@@ -11,18 +15,10 @@ log = logging.getLogger(__name__)
 
 import numpy as np
 import pandas as pd
+import datetime
 
 import resqpy.olio.keyword_files as kf
 
-# functions defined here:
-# def increment_complaints(keyword):
-# def known_keyword(keyword):
-# def add_unknown_keyword(keyword):
-# def default_value(keyword):
-# def complaints(keyword):
-# def required_out_list():
-# def length_unit_conversion_applicable(keyword):
-
 # nexus wellspec columns as required by pagoda
 wk_unknown = -3
 wk_banned = -2
@@ -238,27 +234,37 @@ def load_wellspecs(
     wellspec_file: str,
     well: Optional[str] = None,
     column_list: Union[List[str], None] = [],
+    keep_duplicate_cells: bool = False,
+    keep_null_columns: bool = True,
+    last_data_only: bool = True,
 ) -> Dict[str, Union[pd.DataFrame, None]]:
     """Reads the Nexus wellspec file returning a dictionary of well name to pandas dataframe.
 
     Args:
-       wellspec_file (str): file path of ascii input file containing wellspec keywords.
-       well (str, optional): if present, only the data for the named well are loaded. If None, data
-          for all wells are loaded.
-       column_list (List[str]/None): if present, each dataframe returned contains these
-          columns, in this order. If None, the resulting dictionary contains only well names as keys
-          (each mapping to None rather than a dataframe). If an empty list (default), each dataframe contains
-          the columns listed in the corresponding wellspec header, in the order found in the file.
+        wellspec_file (str): file path of ascii input file containing wellspec keywords.
+        well (str, optional): if present, only the data for the named well are loaded. If None, data
+            for all wells are loaded.
+        column_list (List[str]/None): if present, each dataframe returned contains these
+            columns, in this order. If None, the resulting dictionary contains only well names as keys
+            (each mapping to None rather than a dataframe). If an empty list (default), each dataframe
+            contains the columns listed in the corresponding wellspec header, in the order found in
+            the file.
+        keep_duplicate_cells (bool): if True (default), duplicate cells are kept, otherwise only the
+            last entry is kept.
+        keep_null_columns (bool): if True (default), columns that contain all NA values are kept,
+            otherwise they are removed.
+        last_data_only (bool): If True, only the last entry of well data in the file are used in the
+            dataframe, otherwise all of the well data are used at different times.
 
     Returns:
        well_dict (Dict[str, Union[pd.DataFrame, None]]): mapping each well name found in the
           wellspec file to a dataframe containing the wellspec data.
     """
-    assert wellspec_file, "no wellspec file specified"
+    assert wellspec_file, "No wellspec file specified."
 
     if column_list is not None:
         for column in column_list:
-            assert (column.upper() in wellspec_dict), "unrecognized wellspec column name " + str(column)
+            assert (column.upper() in wellspec_dict), f"Unrecognized wellspec column name {column}."
     selecting = bool(column_list)
 
     well_dict = {}
@@ -270,39 +276,109 @@ def load_wellspecs(
 
     with open(wellspec_file, "r") as file:
         if well:
-            well_data = get_well_data(file, well, well_pointers[well], column_list, selecting)
+            well_data = get_all_well_data(
+                file,
+                well,
+                well_pointers[well],
+                column_list,
+                selecting,
+                keep_duplicate_cells,
+                keep_null_columns,
+                last_data_only,
+            )
             if well_data is not None:
                 well_dict[well] = well_data
         else:
             for well_name, pointer in well_pointers.items():
-                well_data = get_well_data(file, well_name, pointer, column_list, selecting)
+                well_data = get_all_well_data(
+                    file,
+                    well_name,
+                    pointer,
+                    column_list,
+                    selecting,
+                    keep_duplicate_cells,
+                    keep_null_columns,
+                    last_data_only,
+                )
                 if well_data is not None:
                     well_dict[well_name] = well_data
 
     return well_dict
 
 
-def get_well_pointers(wellspec_file: str) -> Dict[str, int]:
+def get_well_pointers(
+    wellspec_file: str,
+    usa_date_format: bool = False,
+    no_date_replacement: Optional[datetime.date] = None,
+) -> Dict[str, List[Tuple[int, Union[None, str]]]]:
     """Gets the file locations of each well in the wellspec file for optimised processing of the data.
 
     Args:
         wellspec_file (str): file path of ascii input file containing wellspec keywords.
+        usa_date_format (bool): if True, the date taken from the wellspec file is in the format
+            MM/DD/YYYY, otherwise it is in the format DD/MM/YYYY.
+        no_date_replacement (datetime.date, optional): if there is no date given for a well, this
+            date is used.
 
     Returns:
-        well_pointers (Dict[str, int]): mapping each well name found in the wellspec file to their
-            file location.
+        well_pointers (Dict[str, List[Tuple[int, None/str]]]): mapping each well name found in
+            the wellspec file to a list of their file locations and dates as tuples. If there is no
+            date before the well data in the file, the date is None. If there is a FileNotFoundError
+            then None is returned.
     """
-    well_pointers = {}
+    well_pointers: Dict[str, List[Tuple[int, Union[None, str]]]] = {}
+    try:
+        with open(wellspec_file, "r") as file:
+            while True:
+                found = kf.find_keyword(file, "WELLSPEC")
+                if not found:
+                    break
+                line = file.readline()
+                words = line.split()
+                assert len(words) >= 2, "Missing well name after WELLSPEC keyword."
+                well_name = words[1]
+                if well_name in well_pointers:
+                    well_pointers[well_name].append((file.tell(), None))
+                else:
+                    well_pointers[well_name] = [(file.tell(), None)]
+    except FileNotFoundError:
+        raise FileNotFoundError(f"The file {wellspec_file} can't be found.")
+
+    time_pointers = {}
     with open(wellspec_file, "r") as file:
         while True:
-            found = kf.find_keyword(file, "WELLSPEC")
+            found = kf.find_keyword(file, "TIME")
             if not found:
                 break
             line = file.readline()
             words = line.split()
-            assert len(words) >= 2, "missing well name after WELLSPEC keyword"
-            well_name = words[1]
-            well_pointers[well_name] = file.tell()
+            assert len(words) >= 2, "Missing date after TIME keyword."
+            date = words[1]
+            try:
+                if usa_date_format:
+                    date_obj = datetime.datetime.strptime(date, "%m/%d/%Y").date()
+                else:
+                    date_obj = datetime.datetime.strptime(date, "%d/%m/%Y").date()
+                if no_date_replacement is not None and date_obj < no_date_replacement:
+                    raise ValueError(
+                        f"The Zero Date {no_date_replacement} must be before the first wellspec TIME {date_obj}.")
+                date = date_obj.isoformat()
+            except ValueError:
+                raise ValueError(f"The date found '{date}' does not match the correct format (usa_date_format "
+                                 f"is {usa_date_format}).")
+            time_pointers[file.tell()] = date
+
+    current_date = None  # Before first TIME
+    if no_date_replacement is not None:
+        current_date = no_date_replacement.isoformat()
+    for well, well_pointer_list in well_pointers.items():
+        for i, well_pointer in enumerate(well_pointer_list):
+            for time_pointer, date in time_pointers.items():
+                if well_pointer[0] > time_pointer:
+                    current_date = date
+                else:
+                    break
+            well_pointers[well][i] = (well_pointer[0], current_date)
 
     return well_pointers
 
@@ -313,11 +389,13 @@ def get_well_data(
     pointer: int,
     column_list: List[str] = [],
     selecting: bool = False,
-    keep_duplicates: bool = True,
+    keep_duplicate_cells: bool = True,
+    keep_null_columns: bool = True,
+    date: Optional[str] = None,
 ) -> Union[pd.DataFrame, None]:
-    """Creates a dataframe of the well data for a given well name in the wellspec file.
+    """Creates a dataframe of the well data for a given well name and at a specific time in the wellspec file.
 
-    The pointer argument is used to go to the file location where the well data is located.
+    The pointer argument is used to go to the file location where the well dataset is located.
 
     Args:
         file (TextIO): the opened wellspec file object.
@@ -325,16 +403,21 @@ def get_well_data(
         pointer (int): the file object's start position of the well data represented as number of
             bytes from the beginning of the file.
         column_list (List[str]): if present, each dataframe returned contains these
-            columns, in this order. If None, the resulting dictionary contains only well names as keys
-            (each mapping to None rather than a dataframe). If an empty list (default), each dataframe contains
-            the columns listed in the corresponding wellspec header, in the order found in the file.
+            columns, in this order. If None, the resulting dictionary contains only well names as
+            keys (each mapping to None rather than a dataframe). If an empty list (default), each
+            dataframe contains the columns listed in the corresponding wellspec header, in the order
+            found in the file.
         selecting (bool): True if the column_list contains at least one column name, False otherwise
             (default).
-        keep_duplicates (bool): if True (default), duplicate cells are kept, otherwise only the
+        keep_duplicate_cells (bool): if True (default), duplicate cells are kept, otherwise only the
             last entry is kept.
+        keep_null_columns (bool): if True (default), columns that contain all NA values are kept,
+            otherwise they are removed.
+        date (str, optional): the well date which is provided by the get_well_pointers function
+            along with the well pointers. 
 
     Returns:
-        Pandas dataframe of the well data or None if at least one row contains all NA.
+        Pandas dataframe of the well data or None if all the data are NA.
     """
     file.seek(pointer)
     kf.skip_blank_lines_and_comments(file)
@@ -359,7 +442,7 @@ def get_well_data(
             break
         line = kf.strip_trailing_comment(file.readline())
         words = line.split()
-        assert len(words) >= len(columns_present), f"insufficient data in line of wellspec table {well_name} [{line}]"
+        assert len(words) >= len(columns_present), f"Insufficient data in line of wellspec table {well_name} [{line}]."
         if selecting:
             for col_index, col in enumerate(column_list):
                 if column_map[col_index] < 0:
@@ -389,12 +472,106 @@ def get_well_data(
                     all_null = False
 
     if all_null:
-        log.warning(f"Null wellspec data for well {well_name}.")
+        log.warning(f"Null wellspec data for well {well_name}{f' at date {date}' if date is not None else ''}.")
         return None
 
     df = pd.DataFrame(data)
-    if not keep_duplicates and any(df.duplicated(subset = ["IW", "JW", "L"])):
+
+    if not keep_null_columns:
+        df.drop(columns = df.columns[df.isna().all()], inplace = True)
+
+    if not keep_duplicate_cells and any(df.duplicated(subset = ["IW", "JW", "L"])):
         log.warning(f"There are duplicate cells for well {well_name}.")
         df.drop_duplicates(subset = ["IW", "JW", "L"], keep = "last", inplace = True)
 
+    def stat_tranformation(row):
+        if row["STAT"] == "ON":
+            return True
+        else:
+            return False
+
+    if "STAT" in df.columns:
+        df["STAT"] = df.apply(lambda row: stat_tranformation(row), axis = 1)
+
     return df
+
+
+def get_all_well_data(
+    file: TextIO,
+    well_name: str,
+    pointers: List[Tuple[int, Union[None, str]]],
+    column_list: List[str] = [],
+    selecting: bool = False,
+    keep_duplicate_cells: bool = False,
+    keep_null_columns: bool = True,
+    last_data_only: bool = True,
+) -> Union[pd.DataFrame, None]:
+    """Creates a dataframe of all the well data for a given well name in the wellspec file.
+
+    This differs from the get_well_data function in that here multiple datasets for a well are
+    combined into a single dataframe if they exist.
+
+    Args:
+        file (TextIO): the opened wellspec file object.
+        well_name (str): name of the well.
+        pointers (List[Tuple[int, None/str]]): a list of the file object's start position of the
+            well data represented as number of bytes from the beginning of the file and the well's
+            date. If no date existed before the well in the file, the date will be None.
+        column_list (List[str]): if present, each dataframe returned contains these
+            columns, in this order. If None, the resulting dictionary contains only well names as
+            keys (each mapping to None rather than a dataframe). If an empty list (default), each
+            dataframe contains the columns listed in the corresponding wellspec header, in the order
+            found in the file.
+        selecting (bool): True if the column_list contains at least one column name, False otherwise
+            (default).
+        keep_duplicate_cells (bool): if True (default), duplicate cells are kept, otherwise only the
+            last entry is kept.
+        keep_null_columns (bool): if True (default), columns that contain all NA values are kept,
+            otherwise they are removed.
+        last_data_only (bool): If True, only the last entry of well data in the file are used in the
+            dataframe, otherwise all of the well data are used at different times.
+
+    Returns:
+        Pandas dataframe of all well data for a specific well name or None if all the data are NA.
+    """
+    if last_data_only:
+        if column_list is None:
+            return None
+        df = get_well_data(
+            file,
+            well_name,
+            pointers[-1][0],
+            column_list,
+            selecting,
+            keep_duplicate_cells,
+            keep_null_columns,
+            date = pointers[-1][1],
+        )
+        return df
+    else:
+        df_list = []
+        for pointer, date in pointers:
+            df = get_well_data(
+                file,
+                well_name,
+                pointer,
+                column_list,
+                selecting,
+                keep_duplicate_cells,
+                date = date,
+            )
+            if df is None:
+                continue
+
+            df["DATE"] = date
+            df_list.append(df)
+
+        if len(df_list) == 0:
+            return None
+
+        df_combined = pd.concat(df_list, ignore_index = True)
+
+        if not keep_null_columns:
+            df_combined.drop(columns = df_combined.columns[df_combined.isna().all()], inplace = True)
+
+        return df_combined
