@@ -7,7 +7,6 @@ version = '15th November 2021'
 import logging
 
 log = logging.getLogger(__name__)
-log.debug('_import_vdb_ensemble.py version ' + version)
 
 import resqpy.model as rq
 import resqpy.olio.vdb as vdb
@@ -33,14 +32,14 @@ def import_vdb_ensemble(
         create_complete_property_set = False,
         # remaining arguments only used if existing_epc is False
         extent_ijk = None,  # 3 element numpy vector
-        corp_xy_units = 'metres',
-        corp_z_units = 'metres',
+        corp_xy_units = 'm',
+        corp_z_units = 'm',
         corp_z_inc_down = True,
         ijk_handedness = 'right',
         geometry_defined_everywhere = True,
         treat_as_nan = None,
-        resqml_xy_units = 'metres',
-        resqml_z_units = 'metres',
+        resqml_xy_units = 'm',
+        resqml_z_units = 'm',
         resqml_z_inc_down = True,
         shift_to_local = True,
         local_origin_place = 'centre',  # 'centre' or 'minimum'
@@ -74,18 +73,18 @@ def import_vdb_ensemble(
           properties imported; only really useful to differentiate from other properties related to the grid
        extent_ijk (triple int, optional): this and remaining arguments are only used if existing_epc is False; the extent
           is only needed in case automatic determination of the extent fails
-       corp_xy_units (string, default 'metres'): the units of x & y values in the vdb corp data; should be 'metres' or 'feet'
-       corp_z_units (string, default 'metres'): the units of z values in the vdb corp data; should be 'metres' or 'feet'
+       corp_xy_units (string, default 'm'): the units of x & y values in the vdb corp data; should be 'm' (metres) or 'ft' (feet)
+       corp_z_units (string, default 'm'): the units of z values in the vdb corp data; should be 'm' (metres) or 'ft' (feet)
        corp_z_inc_down (boolean, default True): set to True if corp z values are depth; False if elevation
        ijk_handedness (string, default 'right'): set to the handedness of the IJK axes in the Nexus model; 'right' or 'left'
        geometry_defined_everywhere (boolean, default True): set to False if inactive cells do not have valid geometry;
           deprecated - use treat_as_nan argument instead
        treat_as_nan (string, optional): if not None, one of 'dots', 'ij_dots', 'inactive'; controls which inactive cells
           have their geometry set to undefined
-       resqml_xy_units (string, default 'metres'): the units of x & y values to use in the generated resqml grid;
-          should be 'metres' or 'feet'
-       resqml_z_units (string, default 'metres'): the units of z values to use in the generated resqml grid;
-          should be 'metres' or 'feet'
+       resqml_xy_units (string, default 'm'): the units of x & y values to use in the generated resqml grid;
+          should be 'm' (metres) or 'ft' (feet)
+       resqml_z_units (string, default 'm'): the units of z values to use in the generated resqml grid;
+          should be 'm' (metres) or 'ft' (feet)
        resqml_z_inc_down (boolean, default True): set to True if resqml z values are to be depth; False for elevations
        shift_to_local (boolean, default True): if True, the resqml coordinate reference system will use a local origin
        local_origin_place (string, default 'centre'): where to place the local origin; 'centre' or 'minimum'; only
@@ -244,13 +243,17 @@ def import_vdb_ensemble(
                 for keyword in props:
                     if keyword_list is not None and keyword not in keyword_list:
                         continue
-                    if property_kind_list is not None:
-                        prop_kind, _, _ = rp.property_kind_and_facet_from_keyword(keyword)
-                        if prop_kind not in property_kind_list and prop_kind not in ['active', 'region initialization']:
-                            continue
+                    prop_kind, facet_type, facet = rp.property_kind_and_facet_from_keyword(keyword)
+                    if property_kind_list is not None and prop_kind not in property_kind_list and prop_kind not in [
+                            'active', 'region initialization'
+                    ]:
+                        continue
                     prop_import_collection.import_vdb_static_property_to_cache(vdbase,
                                                                                keyword,
-                                                                               realization = realisation)
+                                                                               realization = realisation,
+                                                                               property_kind = prop_kind,
+                                                                               facet_type = facet_type,
+                                                                               facet = facet)
                 if decoarsen:
                     decoarsen_array = prop_import_collection.decoarsen_imported_list()
                     if decoarsen_array is not None:
@@ -295,16 +298,18 @@ def import_vdb_ensemble(
                                 continue
                             if keyword_list is not None and keyword not in keyword_list:
                                 continue
-                            if property_kind_list is not None:
-                                prop_kind, _, _ = rp.property_kind_and_facet_from_keyword(keyword)
-                                if prop_kind not in property_kind_list:
-                                    continue
+                            prop_kind, facet_type, facet = rp.property_kind_and_facet_from_keyword(keyword)
+                            if property_kind_list is not None and prop_kind not in property_kind_list:
+                                continue
                             step_import_collection.import_vdb_recurrent_property_to_cache(
                                 vdbase,
                                 r_timestep_number,
                                 keyword,
                                 time_index = tni,  # index into recur_time_series
-                                realization = realisation)
+                                realization = realisation,
+                                property_kind = prop_kind,
+                                facet_type = facet_type,
+                                facet = facet)
                     if decoarsen_array is not None:
                         step_import_collection.decoarsen_imported_list(decoarsen_array = decoarsen_array)
                     # extend hdf5 with cached arrays for this timestep
